unit Fractal;

interface

uses
Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
ExtCtrls;

type
  Tmandelbrot = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure Image1Mousedown(Sender: TObject; Button: TMouseButton;
    Shift: TShiftState; X, Y: Integer);
    procedure Image1Mousemove(Sender: TObject; Shift: TShiftState; X,
    Y: Integer);
    procedure Formcanresize(Sender: TObject; var Newwidth,
    Newheight: Integer; var Resize: Boolean);
  private
    Xl,Xr,Yo,Yu : Double;
    Zeit : Double;
    function Rechne(Re,Im:Double; const X,Y : Double) : Integer;
    procedure Neuberechnung(Links,Oben,Rechts,Unten : Double);
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
    Mandelbrot: Tmandelbrot;

const Iteration=200;
      Step = 256*256*256 div Iteration; // 16 Mio Farben umrechnen

implementation

{$R *.DFM}

uses Gifimage;

function Tmandelbrot.Rechne(Re,Im:Double; const X,Y : Double) : Integer;

var Reel,Imag:Double;
    Farbe : Integer;

begin
  for Farbe:=0 to Iteration do
  begin
    Reel:=Re*Re;
    Imag:=Im*Im;
    if Reel+Imag>8 then begin Result:=Farbe; Exit; end;
    Im:=Im*Re*2-Y;
    Re:=Reel-Imag-X;
  end;
  Result:=0;
end;

procedure Tmandelbrot.Neuberechnung(Links,Oben,Rechts,Unten : Double);

var X,Y,Dx,Dy : Double;
    Xp,Yp : Integer;

begin
  Caption:=Format('(%f %f) - (%f %f)',[Links,Oben,Rechts,Unten]);
  Zeit:=Frac(Now)*86400;
  Dx:=(Rechts-Links)/ClientWidth;
  Dy:=(Oben-Unten)/ClientHeight;
  Xr:=Rechts; Yo:=Oben;
  Xl:=Links;  Yu:=Unten;
  Y:=Oben;
  for Yp:=0 to ClientHeight do
  with Canvas do
  begin
    X:=Links;
    for Xp:=0 to ClientWidth do
    begin
      Pen.Color:=Rechne(X,Y,X,Y)*Step;
      MoveTo(Xp,Yp);
      LineTo(Xp+1,Yp);
      X:=X+Dx;
    end;
    Y:=Y-Dy;
  end;
  Zeit:=(Frac(Now)*86400)-Zeit;
end;

procedure Tmandelbrot.FormCreate(Sender: TObject);

begin
  Xl:=-0.5;
  Xr:=2.5;
  Yu:=1.2;
  Yo:=-1.2;
  cursor:=crhandpoint;
end;

procedure Tmandelbrot.Image1Mousedown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

var X1,Y1 : Double;
    Xdiff,Ydiff : Double;

begin
  Xdiff:=Xr-Xl;
  Ydiff:=Yu-Yo;
  X1:=((X/ClientWidth)*Xdiff)+Xl;
  Y1:=((Y/ClientHeight)*Ydiff)+Yo;
  if Button=Mbleft then
  Neuberechnung(X1-Xdiff/9,Y1-Ydiff/9,X1+Xdiff/9,Y1+Ydiff/9);
  if Button=Mbright then
  Neuberechnung(X1-Xdiff,Y1-Ydiff,X1+Xdiff,Y1+Ydiff);
  if Button=Mbmiddle then
  Neuberechnung(X1-Xdiff/1.5,Y1-Ydiff/1.5,X1+Xdiff/1.5,Y1+Ydiff/1.5);
end;


procedure Tmandelbrot.Image1Mousemove(Sender: TObject; Shift: TShiftState; X, Y: Integer);

var X1,Y1: Double;

begin
  X1:=(X/ClientWidth)*(Xr-Xl)+Xl;
  Y1:=(Y/ClientHeight)*(Yu-Yo)+Yo;
  Caption:=Format('%f sec (%f %f)',[Zeit,X1,Y1]);
end;

procedure Tmandelbrot.Formcanresize(Sender: TObject; var Newwidth,
Newheight: Integer; var Resize: Boolean);

var Xdiff,Ydiff : Double;

begin
  Xdiff:=Xr-Xl;
  Ydiff:=Yu-Yo;
  Xr:=Xl+(Xdiff*ClientWidth)/Newwidth;
  Yu:=Yo+(Ydiff*ClientHeight)/Newheight;
  Neuberechnung(Xl,Yo,Xr,Yu);
  Resize:=True;
end;

end.
